#ifndef _NIXIECLOCK_H
#define _NIXIECLOCK_H

#include "NixieDisplay.H"
#include <FL/Fl_Menu_Button.H>

class NixieClock : public NixieDisplay {
    char  _timestr[256];
    int   _verbose;
    int   _above;
    int   _geometryforce;
    int   _border;
    float _tickspeed;
    char  _errmsg[512];

    // MENU CALLBACKS
    static void AlwaysTopWindow_CB(Fl_Widget *w, void *userdata);
    static void RegularWindow_CB(Fl_Widget *w, void *userdata);
    static void Quit_CB(Fl_Widget *w, void *userdata);
    static void TimeFormatDefault_CB(Fl_Widget *w, void *userdata);
    static void TimeFormat_HM_CB(Fl_Widget *w, void *userdata);
    static void TimeFormat_HMS_CB(Fl_Widget *w, void *userdata);
    static void TimeFormat_HM_MDY_CB(Fl_Widget *w, void *userdata);
    static void TimeFormat_MDY_HM_CB(Fl_Widget *w, void *userdata);
    static void TimeFormat_A_MDY_HM_CB(Fl_Widget *w, void *userdata);
    static void TimeFormatUnix_CB(Fl_Widget *w, void *userdata);
    static void TimeFormatLocale_CB(Fl_Widget *w, void *userdata);

    // OTHER CALLBACKS
    static void Tick_CB(void *data);

    void Init();
    int TimeStringWidth();
    int Load(const char *filename);
    int Save(const char *filename);
    int Exists(const char *filename);
    int GetSettingsFilename(char *filename, int filenamesz);
    const char *GetTimeStr() const { return(_timestr); }

public:
    // CTOR
    NixieClock(int X,int Y,int W,int H,const char*L):NixieDisplay(X,Y,W,H,L) {
        Init();
    }
    // CTOR
    NixieClock(int W,int H,const char*L):NixieDisplay(W,H,L) {
        Init();
    }

    void SetVerbose(int val);
    void SetGeometryForce(int val);
    void SetTickSpeed(float val);
    void SetBorder(int val);
    void SetAbove(int val);
    void SetTimeString(const char *val);
    void UpdateTime();

    int GetAbove() const { return(_above); }

    int LoadSettings();
    int SaveSettings();

    int handle(int e);

    const char *GetErrmsg() const {
        return(_errmsg);
    }
};

#endif
